package trees;

import dictionary.*;
import java.util.NoSuchElementException;

public class TreeIterator<D> implements Iterator<D> {

   static final int NONE = 0;
   static final int UP = 1;
   static final int LEFT = 2;
   static final int RIGHT = 3;
   public static final int PREORDER = UP;
   public static final int INORDER = LEFT;
   public static final int POSTORDER = RIGHT;

   int direction;   // from which direction ...
   Node<D> current; // ... did we enter current node
   int order;       // traversal order

   public TreeIterator(Node<D> root, int order) {
      this.order = order;
      this.current = root;
      this.direction = UP;
      walkNext(order);
   }

   void walkNext(int stopAt) {
      while (current != null && direction != stopAt) {
         switch (direction) {
         case UP: // first time we visit the node
            if (current.left == null) {
               direction = LEFT;
            } else {
               current = current.left;
            }
            break;
         case LEFT: // second time we visit the node
            if (current.right == null) {
               direction = RIGHT;                    
            } else {
               current = current.right;
               direction = UP;
            }
            break;
         case RIGHT: // third and last time we visit this node
            Node<D> previous = current;
            current = current.parent;
            if (current != null) {
               if (current.left == previous) {
                  direction = LEFT;
               } else {
                  direction = RIGHT;
               } 
            }
            break;
         }
         stopAt = order;
      }
   }

   public boolean hasNext() {      
      return current != null;
   }

   public Entry<D> next() {
      if (current == null) {
         throw new NoSuchElementException();
      } else {
         Entry<D> entry = new Entry<D>(current.key, current.data);
         walkNext(NONE);
         return entry;
      }
   }

   public void remove() {
      throw new UnsupportedOperationException();      
   }

}